import numpy

def clip_clim_shape(dsfile):
    import xarray
    import os
    os.environ["PROJ_LIB"] = "%s/anaconda3/envs/xarray/share/proj" %(os.environ["HOME"])
    import fiona
    from shapely.geometry import shape, Point, Polygon
    #
    mskfile = "clim/CERRA_modelling-focus_mask.npy"
    shpfile = "clim/modelling-focus.shp"
    #
    ds = xarray.open_dataset("%s.nc" %(dsfile))
    #
    if os.path.isfile(mskfile):
        mask = numpy.load(mskfile)
    elif os.path.isfile(shpfile):
        # Load the shapefile
        with fiona.open(shpfile, "r") as shp:
            shapes = [shape(feature["geometry"]) for feature in shp]
        # Extract lat/lon arrays (assuming they are 2D)
        lons, lats = ds['longitude'].values, ds['latitude'].values
        #
        # Convert lat/lon grid cells into shapely Point geometries
        mask = numpy.zeros(lons.shape, dtype=bool)
        # Iterate over each grid cell and check if it's inside the shapefile geometry
        for i in range(lons.shape[0]):
            for j in range(lons.shape[1]):
                point = Point(lons[i, j], lats[i, j])
                mask[i, j] = any(poly.contains(point) for poly in shapes)
        numpy.save(mskfile,mask)
    else:
        print("Error: Mask file or shape file not found!")
        ds.close()
        return None
    #
    # Apply mask to dataset using xarray.where()
    msk = ds['longitude'].copy()
    msk[:] = mask
    clipped_ds = ds.where(msk>0, drop=True)
    # Save the clipped dataset
    clipped_ds.to_netcdf("%s_clipped.nc" %(dsfile))
    #
    ds.close()
    clipped_ds.close()

def clip_clim_lonlat(ds, lat_min, lon_min, lat_max, lon_max):
    #
    lons = numpy.array(ds.longitude.copy())
    lons[lons>180.0] -= 360.0
    #
    mask = (lons >= lon_min) & (lons <= lon_max) & (ds.latitude >= lat_min) & (ds.latitude <= lat_max)
    return ds.where(mask, drop=True)

def download():
    import cdsapi
    client = cdsapi.Client()
    #
    for year in [
            "2015", "2016", "2017",
            "2018", "2019", "2020"
        ]:
        for var in [
                "2m_relative_humidity",
                "2m_temperature"
            ]:
            dataset = "reanalysis-cerra-single-levels"
            request = {
                "variable": [var],
                "level_type": "surface_or_atmosphere",
                "data_type": ["reanalysis"],
                "product_type": "analysis",
                "year": [year],
                "month": ["%.2d" %n for n in range(13)[1:]],
                "day": ["%.2d" %n for n in range(32)[1:]],
                "time": ["%.2d:00" %(n*3) for n in range(8)],
                # "area": [20.0, 18.7, 44.1, 45.5], # lat.min, lon.min, lat.max, lon.max
                "data_format": "netcdf"
            }
            fileout = "clim/CERRA/reanalysis-cerra-single-levels-%s-%s.nc" %(var,year)
            #
            client.retrieve(dataset, request, fileout)
    #
    for year in [
            "2015", "2016", "2017",
            "2018", "2019", "2020"
        ]:
        dataset = "reanalysis-cerra-land"
        request = {
            "variable": ["total_precipitation"],
            "level_type": ["surface"],
            "product_type": ["analysis"],
            "year": [year],
            "month": ["%.2d" %n for n in range(13)[1:]],
            "day": ["%.2d" %n for n in range(32)[1:]],
            "time": ["06:00"],
            # "area": [20.0, 18.7, 44.1, 45.5], # lat.min, lon.min, lat.max, lon.max
            "data_format": "netcdf"
        }
        fileout = "clim/CERRA/reanalysis-cerra-land-total_precipitation-%s.nc" %(year)
        #
        client.retrieve(dataset, request, fileout)

# Usage
if __name__ == "__main__":
    if False:
        download()
    #
    if True:
        tasks = ["clim/CERRA/reanalysis-cerra-land-total_precipitation-%d" %(year) for year in range(2015,2021)]
        tasks += ["clim/CERRA/reanalysis-cerra-single-levels-%s-%d" %(var,year) for var in ['2m_temperature','2m_relative_humidity'] for year in range(2015,2021)]
        for task in tasks:
            clip_clim_shape(task)